#include "Graphics.h"
#include <math.h>

Graphics g;
int W = 300; // Width
int H = 300;  // Height

/*
  Partial differential equation: Solving the wave equation in 2d
  Nicolaas E. Groeneboom, FYS4150 H06
  leuat@irio.co.uk
  http://www.irio.co.uk
*/


// This method creats a 2d gaussian clock-wave at position x and y...
void CreatePointWave(double* surf, double posx, double posy, double w, double h)
{ 
 for (int i=0;i<W;i++)
 for (int j=0;j<H;j++)
    {
      double x = -(5.0) + (i-posx)*(10.0/W);
      double y = -(5.0) + (j-posy)*(10.0/H);
      
      double t =exp(-(x*x + y*y)*w)*-h*0.17;
      surf[i + j*W] = surf[i+j*W] + t;
    }
 
}


int main(int a, char *b[])
{
  // Opens a window
  g.openWindow("Wave schmave",W,H);
  
  // Allocate 3 buffers: the solution, 1st and 2nd differentiated solutions
  double* w1 = new double[W*H];  double* w2 = new double[W*H];  double* w3 = new double[W*H];
  double* tmp = 0;  double* wa = w1;  double* wb = w2;  double* wc = w3;
  
  // Wavenumber: must be <1 in order to be stable. Experiment with k, let  
  // k(x) be dependent on x.. looks nice. Notice the reflection of the wave where k changes.
  double k = 0.9;

  printf("Click the screen to drop a wave. Press any key to exit...\n");

  // This variable maintains events
  SDL_Event event;

  // run forever with me!
  bool quit = false;
  while(!quit)
    {
      // Clears the screen (0=black, 255=white)
      g.Clear(0);

      // Draws the solution, pixel by pixel...
      for (int i=0;i<W;i++)
	for (int j=0;j<H;j++) {
	  int c = (int)((wa[i+j*W]*500.0)) + 100;
	  if (c<0) c=-c;
	  if (c>255) c = 255;
	  g.pixel(i,j,c,c,c);
	}


      // The main routine that calculates the next solution

      // Calculate Wave
      double k =0.5;
      for (int i=1;i<W-1;i++)
	for (int j=1;j<H-1;j++)
	{
	  wa[i+j*W] = k* ( -4* wb[i+j*W] + wb[i-1 + j*W] + wb[i+1 + j*W] + wb[i + (j-1)*W] + wb[i + (j+1)*W]) 
	    - wc[i+j*W]+ 2.0*wb[i+j*W]; 

	}
      
      // Nice trick: permute the solutions to save memory
      tmp = wc; wc = wb; wb=wa; wa=tmp;
      
      // Present to the screen
      g.flip();
      
      // Check if keys are pressed
      while ( SDL_PollEvent(&event) ) {
	        switch (event.type) {
 	              case SDL_QUIT:
	                   quit=true;
                  case SDL_KEYDOWN:
	                   quit=true; // guess what happens here
		case SDL_MOUSEBUTTONDOWN: // Drop a wave..
		  CreatePointWave(wb, event.button.x -W/2,event.button.y-H/2  ,50.0,-1.0);
		  break;
		}

	  
      }
    }

  // Clean up that memory!
  delete[] w1;  delete[] w2;  delete[] w3; 
  return 0;
}
